﻿using System.ComponentModel.Composition;

namespace PluginPerceptionTests {
	// These three exports are necessary for the plugin to be detected.
	// Of course, the plugin then also has to implement the ISquadManagerPlugin interface.
	// If you want to be able to write information into a debug log window, also implement
	// the ISquadManagerPluginDebug interface.
	[Export(typeof(SquadManagerPluginInterfaces.ISquadManagerPlugin))]
	[ExportMetadata("Name", "Wahrnehmungsproben")]
	[ExportMetadata("Author", "Ralf Stauder")]
#if DEBUG
	public class PluginPerceptionBase: SquadManagerPluginInterfaces.ISquadManagerPlugin, SquadManagerPluginInterfaces.ISquadManagerPluginDebug {
#else
	public class PluginPerceptionBase: SquadManagerPluginInterfaces.ISquadManagerPlugin {
#endif
		PluginPerceptionBase() {
			// preferably have an interface prepared
			_control = new PluginPerceptionUI(this);
		}

		private System.Windows.Forms.UserControl _control = null;
		// The main UI will request this interface when the plugin is opened.
		public System.Windows.Forms.UserControl GetUI() { return _control; }

		private string _lowerLanguageName;
		internal string LowerLanguageName { get { return _lowerLanguageName; } }
		// this will be called once when the plugin is loaded into the list of available plugins.
		// It's a good idea to store the culture information somehow in case you want to translate
		// other things later, too.
		public string GetLocalizedName(System.Globalization.CultureInfo culture) {
			_lowerLanguageName = culture.Name.Substring(0, 2).ToLower();
			if(_lowerLanguageName == "de")
				return "Wahrnehmungsproben";
			else
				return "Perception tests";
		}

		private SquadManagerPluginInterfaces.ISquadManagerPluginCallback _cb = null;
		internal SquadManagerPluginInterfaces.ISquadManagerPluginCallback Callback { get { return _cb; } }
		// When the plugin is loaded, it get's this callback object that enabled communication with the main
		// application to request information about characters and their details.
		public void ReceiveCallbackInterface(SquadManagerPluginInterfaces.ISquadManagerPluginCallback cb) { _cb = cb; }

		// we don't care about these events, so we always return FALSE.
		public bool EventGlitchHappened(SquadManagerPluginInterfaces.ISquadManagerCharacterInterface actor, bool crit, string context) { return false; }
		public bool EventDamageSuffered(SquadManagerPluginInterfaces.ISquadManagerCharacterInterface actor) { return false; }

#if DEBUG
		// by hiding the debug interface members in a conditional block, we can directly switch between building the full or the
		// debug version of our plugin by changing the build configuration, just like you normally would in Visual Studio.
		internal SquadManagerPluginInterfaces.ISquadManagerPluginDebugCallback _dbgCb = null;
		public void ReceiveDebugCallbackInterface(SquadManagerPluginInterfaces.ISquadManagerPluginDebugCallback cb) { _dbgCb = cb; }
#endif
	}
}
