﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;
using SquadManagerPluginInterfaces;

namespace PluginPerceptionTests {
	public partial class PluginPerceptionUI: UserControl {
		public PluginPerceptionUI(PluginPerceptionBase plugin) {
			// The UI should not have any elements for closing the window, since this will be provided
			// by the frame, this plugin will be displayed in.
			InitializeComponent();
			pluginBase = plugin;
		}
		private PluginPerceptionBase pluginBase;

		private void buttonRoll_Click(object sender, EventArgs e) {
			// Usually it's not good practice to do calculations in the callback functions of UI elements,
			// but for sake of simplicity we still do it here. Preferably this function should only collect
			// UI settings (like the variables "mod" and "onlyNPCs" below) and do everything else
			// in a seperated thread to avoid blocking the UI if something takes longer or accidentally
			// waits for additional user input...
			int mod = System.Convert.ToInt32(numMod.Value);
			bool onlyNPCs = checkOnlyNPCs.Checked;

			string hitName = "hits";
			string glitchName = "+ glitch!";
			// remember to have the plugin at least in English,
			// due to the current user base preferably bilingual English/German.
			if(pluginBase.LowerLanguageName == "de") {
				hitName = "Erfolge";
				glitchName = "+ Patzer!";
			}

#if DEBUG
			// Helpful debug information can be logged into a window, that will automatically pop open if used.
			pluginBase._dbgCb.Log("Name\tType\tPool");
#endif

			// This is one of several function to get character interfaces.
			// This will return every actor in the list (including non-characters, e.g. personas!),
			// while there are also ways to just get the currently selected actor.
			List<ISquadManagerCharacterInterface> list = pluginBase.Callback.GetAllCharacters();
			List<KeyValuePair<int, string>> results = new List<KeyValuePair<int, string>>(list.Count);
			foreach(ISquadManagerCharacterInterface character in list) {
				if(onlyNPCs && !character.NPC)
					continue;

				// Search for the skill you want to use by English name.
				// "Non-characters" like personas or drones won't have the skill, so their pool will be 0.
				int pool = character.GetPool("Perception");
				if(pool > 0) {
					// Getting a test result this way will always be a "hidden roll", even for PCs.
					// If you want to have PCs roll their own results, your UI has to provide fields for
					// that and you need to switch the cases yourself.
					TestResultSuccess res = character.GetTestResult(pool + mod);

					string msg = string.Format("{0}:\t{1} {2} {3}", character.Name, res.Hits, hitName, (res.Glitch) ? (glitchName) : (""));
					results.Add(new KeyValuePair<int, string>(res.Hits, msg));
				}

#if DEBUG
				// Helpful debug information can be logged into a window, that will automatically pop open if used.
				pluginBase._dbgCb.Log(string.Format("{0}\t{1}\t{2}", character.Name, character.Type, pool));
#endif
			}
#if DEBUG
			pluginBase._dbgCb.Log("===========================");
#endif

			results.Sort((KeyValuePair<int, string> a, KeyValuePair<int, string> b) => { return b.Key - a.Key; });

			string[] lines = new string[results.Count];
			for(int i = 0; i < results.Count; i++)
				lines[i] = results[i].Value;
			listResults.Invoke(new Action(delegate() {
				listResults.Items.Clear();
				listResults.Items.AddRange(lines);
			}));
		}
	}
}
